@@ -12,9 +12,11 @@ updating the file tree explorer associated with a project
 """
 
 # Standard library imports
+import configparser
 import os.path as osp
 import shutil
 import functools
+from collections import OrderedDict
 
 # Third party imports
 from qtpy.compat import getexistingdirectory
@@ -22,34 +24,63 @@ from qtpy.QtCore import Signal, Slot
 from qtpy.QtWidgets import QInputDialog, QMenu, QMessageBox, QVBoxLayout
 
 # Local imports
-from spyder.config.base import _, get_home_dir
+from spyder.api.exceptions import SpyderAPIError
+from spyder.api.translations import get_translation
+from spyder.api.plugins import Plugins, SpyderPluginWidget
+from spyder.config.base import (get_home_dir, get_project_config_folder,
+                                running_under_pytest)
 from spyder.config.manager import CONF
-from spyder.api.plugins import SpyderPluginWidget
 from spyder.py3compat import is_text_string, to_text_string
 from spyder.utils import encoding
-from spyder.utils import icon_manager as ima
+from spyder.utils.icon_manager import ima
 from spyder.utils.qthelpers import add_actions, create_action, MENU_SEPARATOR
 from spyder.utils.misc import getcwd_or_home
+from spyder.plugins.projects.api import (BaseProjectType, EmptyProject,
+                                         WORKSPACE)
 from spyder.plugins.projects.utils.watcher import WorkspaceWatcher
 from spyder.plugins.projects.widgets.explorer import ProjectExplorerWidget
 from spyder.plugins.projects.widgets.projectdialog import ProjectDialog
-from spyder.plugins.projects.projecttypes import EmptyProject
-from spyder.plugins.completion.languageserver import (
-    LSPRequestTypes, FileChangeType, WorkspaceUpdateKind)
+from spyder.plugins.completion.api import (
+    CompletionRequestTypes, FileChangeType, WorkspaceUpdateKind)
 from spyder.plugins.completion.decorators import (
     request, handles, class_register)
 
 
+# Localization
+_ = get_translation("spyder")
+
+
 @class_register
 class Projects(SpyderPluginWidget):
     """Projects plugin."""
 
     CONF_SECTION = 'project_explorer'
     CONF_FILE = False
-    sig_pythonpath_changed = Signal()
-    sig_project_created = Signal(object, object, object)
+
+    # This is required for the new API
+    NAME = 'project_explorer'
+    REQUIRES = []
+    OPTIONAL = [Plugins.Completions, Plugins.IPythonConsole, Plugins.Explorer]
+
+    # Signals
+    sig_project_created = Signal(str, str, object)
+    """
+    This signal is emitted to request the Projects plugin the creation of a
+    project.
+
+    Parameters
+    ----------
+    project_path: str
+        Location of project.
+    project_type: str
+        Type of project as defined by project types.
+    project_packages: object
+        Package to install. Currently not in use.
+    """
+
     sig_project_loaded = Signal(object)
     sig_project_closed = Signal(object)
+    sig_pythonpath_changed = Signal()
 
     def __init__(self, parent=None):
         """Initialization."""
@@ -74,6 +105,7 @@ class Projects(SpyderPluginWidget):
         self.completions_available = False
         self.explorer.setup_project(self.get_active_project_path())
         self.watcher.connect_signals(self)
+        self._project_types = OrderedDict()
 
     #------ SpyderPluginWidget API ---------------------------------------------
     def get_plugin_title(self):
@@ -129,55 +161,81 @@ class Projects(SpyderPluginWidget):
         """Register plugin in Spyder's main window"""
         ipyconsole = self.main.ipyconsole
         treewidget = self.explorer.treewidget
-        lspmgr = self.main.completions
+        completions = self.main.get_plugin(Plugins.Completions, error=False)
+        outlineexplorer = self.main.get_plugin(
+            Plugins.OutlineExplorer, error=False)
 
         self.add_dockwidget()
-        self.explorer.sig_open_file.connect(self.main.open_file)
-        self.register_widget_shortcuts(treewidget)
+        self.explorer.sig_open_file_requested.connect(self.main.open_file)
 
         treewidget.sig_delete_project.connect(self.delete_project)
-        treewidget.sig_edit.connect(self.main.editor.load)
+        treewidget.sig_open_file_requested.connect(self.main.editor.load)
         treewidget.sig_removed.connect(self.main.editor.removed)
-        treewidget.sig_removed_tree.connect(self.main.editor.removed_tree)
+        treewidget.sig_tree_removed.connect(self.main.editor.removed_tree)
         treewidget.sig_renamed.connect(self.main.editor.renamed)
-        treewidget.sig_renamed_tree.connect(self.main.editor.renamed_tree)
-        treewidget.sig_create_module.connect(self.main.editor.new)
-        treewidget.sig_new_file.connect(
+        treewidget.sig_tree_renamed.connect(self.main.editor.renamed_tree)
+        treewidget.sig_module_created.connect(self.main.editor.new)
+        treewidget.sig_file_created.connect(
             lambda t: self.main.editor.new(text=t))
-        treewidget.sig_open_interpreter.connect(
+        treewidget.sig_open_interpreter_requested.connect(
             ipyconsole.create_client_from_path)
-        treewidget.redirect_stdio.connect(
+        treewidget.sig_redirect_stdio_requested.connect(
             self.main.redirect_internalshell_stdio)
-        treewidget.sig_run.connect(
+        treewidget.sig_run_requested.connect(
             lambda fname:
             ipyconsole.run_script(fname, osp.dirname(fname), '', False, False,
                                   False, True, False))
 
+        # TODO: This is not necessary anymore due to us starting workspace
+        # services in the editor. However, we could restore it in the future.
+        # completions.sig_language_completions_available.connect(
+        #     lambda settings, language:
+        #         self.start_workspace_services())
+        if completions:
+            completions.sig_stop_completions.connect(
+                self.stop_workspace_services)
+
         # New project connections. Order matters!
         self.sig_project_loaded.connect(
-            lambda v: self.main.workingdirectory.chdir(v))
+            lambda path:
+            self.main.workingdirectory.chdir(
+                directory=path,
+                sender_plugin=self
+            )
+        )
         self.sig_project_loaded.connect(
             lambda v: self.main.set_window_title())
-        self.sig_project_loaded.connect(
-            functools.partial(lspmgr.project_path_update,
-                              update_kind=WorkspaceUpdateKind.ADDITION))
+
+        if completions:
+            self.sig_project_loaded.connect(
+                functools.partial(completions.project_path_update,
+                                  update_kind=WorkspaceUpdateKind.ADDITION,
+                                  instance=self))
         self.sig_project_loaded.connect(
             lambda v: self.main.editor.setup_open_files())
         self.sig_project_loaded.connect(self.update_explorer)
-        self.sig_project_loaded.connect(
-            lambda v: self.main.outlineexplorer.update_all_editors())
+        if outlineexplorer:
+            self.sig_project_loaded.connect(
+                lambda v: self.main.outlineexplorer.update_all_editors())
         self.sig_project_closed[object].connect(
-            lambda v: self.main.workingdirectory.chdir(
-                self.get_last_working_dir()))
+            lambda path:
+            self.main.workingdirectory.chdir(
+                directory=self.get_last_working_dir(),
+                sender_plugin=self
+            )
+        )
         self.sig_project_closed.connect(
             lambda v: self.main.set_window_title())
-        self.sig_project_closed.connect(
-            functools.partial(lspmgr.project_path_update,
-                              update_kind=WorkspaceUpdateKind.DELETION))
+        if completions:
+            self.sig_project_closed.connect(
+                functools.partial(completions.project_path_update,
+                                  update_kind=WorkspaceUpdateKind.DELETION,
+                                  instance=self))
         self.sig_project_closed.connect(
             lambda v: self.main.editor.setup_open_files())
-        self.sig_project_closed.connect(
-            lambda v: self.main.outlineexplorer.update_all_editors())
+        if outlineexplorer:
+            self.sig_project_closed.connect(
+                lambda v: self.main.outlineexplorer.update_all_editors())
         self.recent_project_menu.aboutToShow.connect(self.setup_menu_actions)
 
         self.main.restore_scrollbar_position.connect(
@@ -185,12 +243,17 @@ class Projects(SpyderPluginWidget):
         self.sig_pythonpath_changed.connect(self.main.pythonpath_changed)
         self.main.editor.set_projects(self)
 
-        # Connect to file explorer to keep single click to open files in sync
-        self.main.explorer.fileexplorer.sig_option_changed.connect(
-            self.set_single_click_to_open
-        )
+        self.sig_project_loaded.connect(
+            lambda v: self.main.editor.set_current_project_path(v))
+        self.sig_project_closed.connect(
+            lambda v: self.main.editor.set_current_project_path())
 
-    def set_single_click_to_open(self, option, value):
+        # Connect to file explorer to keep single click to open files in sync
+        # TODO: Remove this once projects is migrated
+        CONF.observe_configuration(self, 'explorer', 'single_click_to_open')
+        self.register_project_type(self, EmptyProject)
+
+    def on_configuration_change(self, option, section, value):
         """Set single click to open files and directories."""
         if option == 'single_click_to_open':
             self.explorer.treewidget.set_single_click_to_open(value)
@@ -261,13 +324,16 @@ class Projects(SpyderPluginWidget):
 
     @Slot()
     def create_new_project(self):
-        """Create new project"""
+        """Create new project."""
         self.unmaximize()
         active_project = self.current_active_project
-        dlg = ProjectDialog(self)
-        dlg.sig_project_creation_requested.connect(self._create_project)
-        dlg.sig_project_creation_requested.connect(self.sig_project_created)
-        if dlg.exec_():
+        dlg = ProjectDialog(self, project_types=self.get_project_types())
+        result = dlg.exec_()
+        data = dlg.project_data
+        root_path = data.get("root_path", None)
+        project_type = data.get("project_type", EmptyProject.ID)
+
+        if result:
             # A project was not open before
             if active_project is None:
                 if self.get_option('visible_if_project_open'):
@@ -277,16 +343,44 @@ class Projects(SpyderPluginWidget):
                 # TODO: Don't emit sig_project_closed when we support
                 # multiple workspaces.
                 self.sig_project_closed.emit(active_project.root_path)
+
+            self._create_project(root_path, project_type_id=project_type)
             self.sig_pythonpath_changed.emit()
             self.restart_consoles()
+            dlg.close()
 
-    def _create_project(self, path):
+    def _create_project(self, root_path, project_type_id=EmptyProject.ID,
+                        packages=None):
         """Create a new project."""
-        self.open_project(path=path)
+        project_types = self.get_project_types()
+        if project_type_id in project_types:
+            project_type_class = project_types[project_type_id]
+            project = project_type_class(
+                root_path=root_path,
+                parent_plugin=project_type_class._PARENT_PLUGIN,
+            )
 
-    def open_project(self, path=None, restart_consoles=True,
-                     save_previous_files=True):
-        """Open the project located in `path`"""
+            created_succesfully, message = project.create_project()
+            if not created_succesfully:
+                QMessageBox.warning(self, "Project creation", message)
+                shutil.rmtree(root_path, ignore_errors=True)
+                return
+
+            # TODO: In a subsequent PR return a value and emit based on that
+            self.sig_project_created.emit(root_path, project_type_id, packages)
+            self.open_project(path=root_path, project=project)
+        else:
+            if not running_under_pytest():
+                QMessageBox.critical(
+                    self,
+                    _('Error'),
+                    _("<b>{}</b> is not a registered Spyder project "
+                      "type!").format(project_type_id)
+                )
+
+    def open_project(self, path=None, project=None, restart_consoles=True,
+                     save_previous_files=True, workdir=None):
+        """Open the project located in `path`."""
         self.unmaximize()
         if path is None:
             basedir = get_home_dir()
@@ -296,20 +390,30 @@ class Projects(SpyderPluginWidget):
             path = encoding.to_unicode_from_fs(path)
             if not self.is_valid_project(path):
                 if path:
-                    QMessageBox.critical(self, _('Error'),
-                                _("<b>%s</b> is not a Spyder project!") % path)
+                    QMessageBox.critical(
+                        self,
+                        _('Error'),
+                        _("<b>%s</b> is not a Spyder project!") % path,
+                    )
                 return
         else:
             path = encoding.to_unicode_from_fs(path)
-        self.add_to_recent(path)
+        if project is None:
+            project_type_class = self._load_project_type_class(path)
+            project = project_type_class(
+                root_path=path,
+                parent_plugin=project_type_class._PARENT_PLUGIN,
+            )
 
         # A project was not open before
         if self.current_active_project is None:
             if save_previous_files and self.main.editor is not None:
                 self.main.editor.save_open_files()
+
             if self.main.editor is not None:
                 self.main.editor.set_option('last_working_dir',
                                             getcwd_or_home())
+
             if self.get_option('visible_if_project_open'):
                 self.show_explorer()
         else:
@@ -322,20 +426,29 @@ class Projects(SpyderPluginWidget):
             # multiple workspaces.
             self.sig_project_closed.emit(
                 self.current_active_project.root_path)
+            self.watcher.stop()
 
-        project = EmptyProject(path)
         self.current_active_project = project
         self.latest_project = project
+        self.add_to_recent(path)
+
         self.set_option('current_project_path', self.get_active_project_path())
 
         self.setup_menu_actions()
-        self.sig_project_loaded.emit(path)
+        if workdir and osp.isdir(workdir):
+            self.sig_project_loaded.emit(workdir)
+        else:
+            self.sig_project_loaded.emit(path)
         self.sig_pythonpath_changed.emit()
         self.watcher.start(path)
 
         if restart_consoles:
             self.restart_consoles()
 
+        open_successfully, message = project.open_project()
+        if not open_successfully:
+            QMessageBox.warning(self, "Project open", message)
+
     def close_project(self):
         """
         Close current project and return to a window without an active
@@ -347,6 +460,11 @@ class Projects(SpyderPluginWidget):
                 self.set_project_filenames(
                     self.main.editor.get_open_filenames())
             path = self.current_active_project.root_path
+            closed_sucessfully, message = (
+                self.current_active_project.close_project())
+            if not closed_sucessfully:
+                QMessageBox.warning(self, "Project close", message)
+
             self.current_active_project = None
             self.set_option('current_project_path', None)
             self.setup_menu_actions()
@@ -541,15 +659,17 @@ class Projects(SpyderPluginWidget):
             path = self.get_active_project_path()
             self.notify_project_open(path)
 
-    def stop_workspace_services(self):
+    def stop_workspace_services(self, _language):
         """Disable LSP workspace functionality."""
         self.completions_available = False
 
     def emit_request(self, method, params, requires_response):
         """Send request/notification/response to all LSP servers."""
-        params['requires_response'] = requires_response
-        params['response_instance'] = self
-        self.main.completions.broadcast_notification(method, params)
+        completions = self.main.get_plugin(Plugins.Completions, error=False)
+        if completions:
+            params['requires_response'] = requires_response
+            params['response_instance'] = self
+            self.main.completions.broadcast_notification(method, params)
 
     @Slot(str, dict)
     def handle_response(self, method, params):
@@ -560,7 +680,7 @@ class Projects(SpyderPluginWidget):
             handler(params)
 
     @Slot(str, str, bool)
-    @request(method=LSPRequestTypes.WORKSPACE_WATCHED_FILES_UPDATE,
+    @request(method=CompletionRequestTypes.WORKSPACE_WATCHED_FILES_UPDATE,
              requires_response=False)
     def file_moved(self, src_file, dest_file, is_dir):
         """Notify LSP server about a file that is moved."""
@@ -584,7 +704,7 @@ class Projects(SpyderPluginWidget):
         }
         return params
 
-    @request(method=LSPRequestTypes.WORKSPACE_WATCHED_FILES_UPDATE,
+    @request(method=CompletionRequestTypes.WORKSPACE_WATCHED_FILES_UPDATE,
              requires_response=False)
     @Slot(str, bool)
     def file_created(self, src_file, is_dir):
@@ -600,7 +720,7 @@ class Projects(SpyderPluginWidget):
         }
         return params
 
-    @request(method=LSPRequestTypes.WORKSPACE_WATCHED_FILES_UPDATE,
+    @request(method=CompletionRequestTypes.WORKSPACE_WATCHED_FILES_UPDATE,
              requires_response=False)
     @Slot(str, bool)
     def file_deleted(self, src_file, is_dir):
@@ -616,7 +736,7 @@ class Projects(SpyderPluginWidget):
         }
         return params
 
-    @request(method=LSPRequestTypes.WORKSPACE_WATCHED_FILES_UPDATE,
+    @request(method=CompletionRequestTypes.WORKSPACE_WATCHED_FILES_UPDATE,
              requires_response=False)
     @Slot(str, bool)
     def file_modified(self, src_file, is_dir):
@@ -632,7 +752,7 @@ class Projects(SpyderPluginWidget):
         }
         return params
 
-    @request(method=LSPRequestTypes.WORKSPACE_FOLDERS_CHANGE,
+    @request(method=CompletionRequestTypes.WORKSPACE_FOLDERS_CHANGE,
              requires_response=False)
     def notify_project_open(self, path):
         """Notify LSP server about project path availability."""
@@ -643,7 +763,7 @@ class Projects(SpyderPluginWidget):
         }
         return params
 
-    @request(method=LSPRequestTypes.WORKSPACE_FOLDERS_CHANGE,
+    @request(method=CompletionRequestTypes.WORKSPACE_FOLDERS_CHANGE,
              requires_response=False)
     def notify_project_close(self, path):
         """Notify LSP server to unregister project path."""
@@ -654,8 +774,8 @@ class Projects(SpyderPluginWidget):
         }
         return params
 
-    @handles(LSPRequestTypes.WORKSPACE_APPLY_EDIT)
-    @request(method=LSPRequestTypes.WORKSPACE_APPLY_EDIT,
+    @handles(CompletionRequestTypes.WORKSPACE_APPLY_EDIT)
+    @request(method=CompletionRequestTypes.WORKSPACE_APPLY_EDIT,
              requires_response=False)
     def handle_workspace_edit(self, params):
         """Apply edits to multiple files and notify server about success."""
@@ -666,3 +786,80 @@ class Projects(SpyderPluginWidget):
             'language': edits['language']
         }
         return response
+
+    # --- New API:
+    # ------------------------------------------------------------------------
+    def _load_project_type_class(self, path):
+        """
+        Load a project type class from the config project folder directly.
+
+        Notes
+        -----
+        This is done directly, since using the EmptyProject would rewrite the
+        value in the constructor. If the project found has not been registered
+        as a valid project type, the EmptyProject type will be returned.
+
+        Returns
+        -------
+        spyder.plugins.projects.api.BaseProjectType
+            Loaded project type class.
+        """
+        fpath = osp.join(
+            path, get_project_config_folder(), 'config', WORKSPACE + ".ini")
+
+        project_type_id = EmptyProject.ID
+        if osp.isfile(fpath):
+            config = configparser.ConfigParser()
+            config.read(fpath)
+            project_type_id = config[WORKSPACE].get(
+                "project_type", EmptyProject.ID)
+
+
+        EmptyProject._PARENT_PLUGIN = self
+        project_types = self.get_project_types()
+        project_type_class = project_types.get(project_type_id, EmptyProject)
+        return project_type_class
+
+    def register_project_type(self, parent_plugin, project_type):
+        """
+        Register a new project type.
+
+        Parameters
+        ----------
+        parent_plugin: spyder.plugins.api.plugins.SpyderPluginV2
+            The parent plugin instance making the project type registration.
+        project_type: spyder.plugins.projects.api.BaseProjectType
+            Project type to register.
+        """
+        if not issubclass(project_type, BaseProjectType):
+            raise SpyderAPIError("A project type must subclass "
+                                 "BaseProjectType!")
+
+        project_id = project_type.ID
+        if project_id in self._project_types:
+            raise SpyderAPIError("A project type id '{}' has already been "
+                                 "registered!".format(project_id))
+
+        project_type._PARENT_PLUGIN = parent_plugin
+        self._project_types[project_id] = project_type
+
+    def get_project_types(self):
+        """
+        Return available registered project types.
+
+        Returns
+        -------
+        dict
+            Project types dictionary. Keys are project type IDs and values
+            are project type classes.
+        """
+        return self._project_types
+
+    # TODO: To be removed after migration
+    def get_plugin(self, plugin_name):
+        """
+        Return a plugin instance by providing the plugin's NAME.
+        """
+        PLUGINS = self.main._PLUGINS
+        if plugin_name in PLUGINS:
+            return PLUGINS[plugin_name]
